import { inject, Injector, runInInjectionContext, provideEnvironmentInitializer } from '@angular/core';
import { toSignal } from '@angular/core/rxjs-interop';
import { Router, ChildrenOutletContexts, NavigationEnd, ActivatedRoute } from '@angular/router';
import { filter, map, startWith, switchMap, combineLatest } from 'rxjs';

/**
 * Acquire a signal of the `ActivatedRoute` object of the
 * direct child route of the current route.
 *
 * @remarks
 * Requires an Injection Context.
 * Depends on `Router` and `ChildrenOutletContexts`.
 *
 * @param outlet the name of the route outlet to target,
 * defaults to name of the default outlet
 */
function useChildActivatedRoute(outlet = 'primary') {
    const router = inject(Router);
    const contexts = inject(ChildrenOutletContexts);
    const value = () => contexts.getOrCreateContext(outlet).route;
    const stream = router.events.pipe(filter((e) => e instanceof NavigationEnd), map(() => value()));
    return toSignal(stream, { initialValue: value() });
}

/**
 * Workaround to emulate an injection context in the route's loadChildren resolver,
 * by retrieving the injector from the nearest `EnvironmentInjector` via a
 * route-level environment initializer.
 *
 * @param route the route config where a injection context is needed for `loadChildren`
 * @param injectorDefault the default injector to use if the environment initializer failed
 * to run, which is possible if no router navigation happens, such as during SSR route discovery.
 * @returns a new route config that can be used in replace of the given route config.
 *
 * @remarks
 * This enables asynchronous providers for routes, allowing dynamically provide
 * injectables based on external factors.
 *
 * @remarks
 * This function does not transform child routes. Invoke on each route that
 * needs an injection context.
 *
 * @example
 *  ```ts
 * export const APP_ROUTES: Routes = [
 *   setupInjectionContextForLoadChildren({
 *     path: '',
 *     loadChildren: async (providerLoader = inject(MyProviderLoader)) => [
 *       {
 *         path: '',
 *         providers: await providerLoader.load(),
 *         children: [
 *           ...
 *         ],
 *       },
 *     ],
 *   }),
 * ];
 *  ```
 *
 * @see https://github.com/angular/angular/issues/51532#issuecomment-1956138610
 * for the original inspiration for this workaround
 */
function setupInjectionContextForLoadChildren(route, injectorDefault) {
    let injectorRoute;
    const injectorInitializer = () => {
        injectorRoute = inject(Injector);
    };
    const transformRoute = (child) => {
        if (!child.loadChildren)
            return child;
        const loadChildren = child.loadChildren;
        child.loadChildren = (...args) => {
            const injector = injectorRoute ?? injectorDefault?.();
            if (!injector)
                throw new Error('missing injector for loadChildren');
            return runInInjectionContext(injector, () => loadChildren(...args));
        };
        return child;
    };
    return {
        path: '',
        providers: [provideEnvironmentInitializer(injectorInitializer)],
        children: [transformRoute(route)],
    };
}

/**
 * Type helper that helps creating valid `RouteDataDefinition` objects
 * utilizing type-inference.
 *
 * @remarks The type of the `value` is guaranteed to be compatible with
 * the declared type in the token.
 *
 * @example
 *  ```ts
 *  export const NAV_STATUS = new RouteDataToken<NavStatus>("NAV_STATUS");
 *  ```
 *  ```ts
 *  defineRouteData({
 *    token: NAV_STATUS,
 *    value: NavStatus.Expanded,
 *  });
 *  ```
 *
 * @see `compileRouteData`
 */
function defineRouteData(def) {
    return def;
}
/**
 * Take a list of `RouteDataDefinition` and compile them into an object
 * that can be accepted as the data field of an Angular route, so that these
 * data records can be retrieved later.
 * @param definitions data records, preferably created via `defineRouteData`
 * @returns an object that can be used as the data of an Angular route
 *
 * @see `defineRouteData`
 * @see `useRouteData`
 *
 * @example
 *  ```ts
 *  export const NAV_STATUS = new RouteDataToken<NavStatus>("NAV_STATUS");
 *  ```
 *  ```ts
 *  export const PAGE_ROUTES: Routes = [
 *    {
 *      path: "",
 *      component: PageComponent,
 *      data: compileRouteData([
 *        defineRouteData({
 *          token: NAV_STATUS,
 *          value: NavStatus.Collapsed,
 *        }),
 *      ]),
 *    },
 *  ]
 *  ```
 */
function compileRouteData(definitions) {
    const data = {};
    for (const def of definitions)
        data[def.token.key] = def.value;
    return data;
}

const TYPE = Symbol('TYPE');
/**
 * Token that declares a type of route data.
 *
 * @remarks The generic type parameter is the key to guaranteed type-safety,
 * and should always be defined.
 *
 * @example
 *  ```ts
 *  export const NAV_STATUS = new RouteDataToken<NavStatus>("NAV_STATUS");
 *  ```
 *
 * @see `defineRouteData`
 */
class RouteDataToken {
    /**
     * @param description Descriptive text for debugging purposes.
     */
    constructor(description) {
        this.description = description;
        /**
         * Property key that can be used to store and access the corresponding value of
         * this token from an route's data object.
         */
        this.key = Symbol(this.description);
    }
}

/**
 * Retrieves the value of the given `RouteDataToken` from the current or deeper
 * routes.
 * @param token the target token to retrieve value of
 * @param scope "current": retrieves from the `ActivatedRoute`; "children": retrieves
 * from the `ActivatedRoute` and all its child routes.
 * @returns signal of value of the given token, or `null` if such value is not
 * defined. Navigation may change the value.
 *
 * @example
 *  ```ts
 *  export const NAV_STATUS = new RouteDataToken<NavStatus>("NAV_STATUS");
 *  ```
 *  ```ts
 *  export const PAGE_ROUTES: Routes = [
 *    {
 *      path: "",
 *      component: PageComponent,
 *      data: compileRouteData([
 *        defineRouteData({
 *          token: NAV_STATUS,
 *          value: NavStatus.Collapsed,
 *        }),
 *      ]),
 *    },
 *  ]
 *  ```
 *  ```ts
 *  private navStatus = useRouteData(NAV_STATUS, "children");
 *  ```
 *  ```html
 *  <app-nav [status]="navStatus()" />
 *  ```
 *
 * @see `readRouteData`
 */
function useRouteData(token, scope) {
    const router = inject(Router);
    const route = inject(ActivatedRoute);
    return toSignal(router.events.pipe(filter((e) => e instanceof NavigationEnd), startWith(null), switchMap(() => {
        if (scope === 'current')
            return route.data;
        return aggregateRouteTreeData(route);
    }), map((data) => data[token.key] ?? null)), { requireSync: true });
}
/**
 * Read the current value of the given token from the data object of a route.
 * @param data the data object from `ActivatedRoute` or `ActivatedRouteSnapshot`
 * @param token the target token to retrieve value of
 * @returns the value, or `undefined` if such value is not defined
 *
 * @example
 *  ```ts
 *  export const NAV_STATUS = new RouteDataToken<NavStatus>("NAV_STATUS");
 *  ```
 *  ```ts
 *  export const PAGE_ROUTES: Routes = [
 *    {
 *      path: "",
 *      component: PageComponent,
 *      data: compileRouteData([
 *        defineRouteData({
 *          token: NAV_STATUS,
 *          value: NavStatus.Collapsed,
 *        }),
 *      ]),
 *    },
 *  ]
 *  ```
 *  ```ts
 *  const route = this.activatedRoute.snapshot;
 *  const value = readRouteData(route.data, NAV_STATUS);
 *  ```
 *
 * @see `useRouteData`
 */
function readRouteData(data, token) {
    return data[token.key];
}
function aggregateRouteTreeData(root) {
    const observables = [root.data];
    for (const child of root.children)
        observables.push(aggregateRouteTreeData(child));
    return combineLatest(observables).pipe(map((data) => Object.assign({}, ...data)));
}

/**
 * Generated bundle index. Do not edit.
 */

export { RouteDataToken, compileRouteData, defineRouteData, readRouteData, setupInjectionContextForLoadChildren, useChildActivatedRoute, useRouteData };
//# sourceMappingURL=angularity-router.mjs.map
